%{
open Syntax
let prim s = Prim(s,None)
%}

%token <string> IDENT
%token <int> INT
%token <int> INPUT
%token <int> OUTPUT
%token PLUS
%token MINUS
%token DIV
%token PAR
%token REP
%token ZERO
%token NU
%token LET
%token IN
%token IF
%token THEN
%token ELSE
%token EQ
%token GT
%token GTEQ
%token LT
%token LTEQ
%token LPAREN
%token RPAREN
%token SLPAREN
%token SRPAREN
%token COMMA
%token EOF
%token COLON
%token PERIOD
%token NIL
%token <string> PRIM
%token <bool> BOOL

%nonassoc IN
%left PAR
%nonassoc ELSE
%nonassoc THEN
%right REP
%nonassoc PERIOD
%left COMMA
%left EQ GT GTEQ LT LTEQ
%left PLUS MINUS
%left MULT DIV
%left APP
%right SIGN

%start main
%type <unit Syntax.procexp> main
%type <unit Syntax.valexp> val
%type <string> pattern

%%

main:
  proc EOF
  { $1 }

proc:
  ZERO
  { Zero }
| IDENT OUTPUT val PERIOD proc
  { Out($1, $3, $5, $2) }
| IDENT OUTPUT val 
  { Out($1, $3, Zero, $2) }
| IDENT INPUT pattern PERIOD proc
  { In($1, $3, (), $5, $2) }
| IDENT INPUT pattern
  { In($1, $3, (), Zero, $2) }
| proc PAR proc
  { Par($1, $3) }  
| REP proc
  { Rep($2) }
| LPAREN proc RPAREN
  { $2 }
| SLPAREN proc SRPAREN
  { Special($2) }
| NU IDENT IN proc
  {Nu($2, (), $4, Parsing.symbol_start())}
| IF val THEN proc ELSE proc
  {If($2, $4, $6, Parsing.symbol_start())}
| LET IDENT EQ val IN proc
  {Let($2, (), $4, $6, Parsing.symbol_start())}
;

val:
  INT
  {Int($1)}
| LPAREN RPAREN
  {Unit}
| IDENT
  {Var($1)}
| BOOL
  {Bool($1)}
| NIL
  {prim "nil"}
| MINUS INT %prec SIGN
  {Int(0-($2))}
| val PLUS val
  {App(prim "+", Pair($1, $3), Parsing.symbol_start())}
| val MINUS val
  {App(prim "-", Pair($1, $3), Parsing.symbol_start())}
| val REP val %prec MULT
  {App(prim "*", Pair($1, $3), Parsing.symbol_start())}
| val DIV val
  {App(prim "/", Pair($1, $3), Parsing.symbol_start())}
| val EQ val
  {App(prim "=", Pair($1, $3), Parsing.symbol_start())}
| val GT val
  {App(prim ">", Pair($1, $3), Parsing.symbol_start())}
| val GTEQ val
  {App(prim ">=", Pair($1, $3), Parsing.symbol_start())}
| val LT val
  {App(prim "<", Pair($1, $3), Parsing.symbol_start())}
| val LTEQ val
  {App(prim "<=", Pair($1, $3), Parsing.symbol_start())}
| PRIM val %prec APP
  {App(prim $1, $2,Parsing.symbol_start())}
| LPAREN val COMMA val RPAREN
  {Pair($2, $4)}
| LPAREN val RPAREN
  {$2}
;

pattern:
  IDENT
  {$1}
;
